// @flow
// $FlowFixMe: Need to ignore the mergeDeepWith import until we are on Immutable@4.0.0
import { List, Map, mergeDeepWith } from 'immutable'
import { toPath } from 'lodash'
import type { Map as ImmutableMap, List as ImmutableList } from 'immutable'

const arrayPattern = /\[(\d+)\]/

const undefinedArrayMerge = (previous, next) =>
  next !== undefined ? next : previous

const mergeLists = (original, value) => {
  if (original && List.isList(original)) {
    if (original.mergeDeepWith) {
      // In this case we are using Immutable.js@3.x since we still have List.mergeDeepWith
      return original.mergeDeepWith(undefinedArrayMerge, value)
    }

    // Immutable 4.x users will end here.
    return mergeDeepWith(undefinedArrayMerge, original, value)
  }

  return value
}

/*
 * ImmutableJS' setIn function doesn't support array (List) creation
 * so we must pre-insert all arrays in the path ahead of time.
 * 
 * Additionally we must also pre-set a dummy Map at the location
 * of an array index if there's parts that come afterwards because 
 * the setIn function uses `{}` to mark an unset value instead of 
 * undefined (which is the case for list / arrays).
 */
export default function setIn(
  state: ImmutableMap<string, *> | ImmutableList<*>,
  field: string,
  value: any
) {
  const path = toPath(field)
  if (!field || typeof field !== 'string' || !arrayPattern.test(field)) {
    return state.setIn(path, value)
  }

  return state.withMutations(mutable => {
    for (let pathIndex = 0; pathIndex < path.length - 1; ++pathIndex) {
      const nextPart = path[pathIndex + 1]
      if (isNaN(nextPart)) {
        continue
      }

      mutable = mutable.updateIn(path.slice(0, pathIndex + 1), value =>
        mergeLists(value, new List().set(parseInt(nextPart, 10), new Map()))
      )
    }

    return mutable.setIn(path, value)
  })
}
